
(* 実数 + 複素数 *)
let ( +< ) x z = 
  let x_c = { Complex.re = x; Complex.im = 0. } in
  Complex.add x_c z

let ( -< ) x z = 
  let x_c = { Complex.re = x; Complex.im = 0. } in
  Complex.sub x_c z

(* 純虚数 *)
let i y = { Complex.re = 0.; Complex.im = y } 

(* 複素数の演算  *)
let ( +@ ) z1 z2 = Complex.add z1 z2
let ( -@ ) z1 z2 = Complex.sub z1 z2
let ( *@ ) z1 z2 = Complex.mul z1 z2

(* リーマン球面 *)
module CP1 : sig
  type t = ComplexPlane of Complex.t | Infty
  val of_c2 : Complex.t * Complex.t -> t
  val to_c2 : t -> Complex.t * Complex.t
end
= struct
  type t =
    | ComplexPlane of Complex.t
    | Infty

  let of_c2 (z1, z2) =
    if z2 <> Complex.zero then ComplexPlane (Complex.div z1 z2)
    else Infty

  let to_c2 z =
    match z with
    | ComplexPlane z -> (z, Complex.one)
    | Infty -> (Complex.one, Complex.zero)
end


