
(* 複素数の演算 *)
let ( +@ ) z1 z2 = Complex.add z1 z2
let ( -@ ) z1 z2 = Complex.sub z1 z2
let ( *@ ) z1 z2 = Complex.mul z1 z2

let pi = 4. *. atan 1.

let exp_2pi m n =
  let theta = 2. *. pi *. (float_of_int n) /. (float_of_int m) in
  { Complex.re = cos theta; Complex.im = sin theta }

(* DFTの通常の計算 *)
let dft_simple x =
  let len = Array.length x in
  let y_arr = Array.init len (fun n -> ((exp_2pi len (-n)) *@ x.(n))) in
  Array.fold_left ( +@ ) Complex.zero y_arr

(* *)
let calc_p2 x n =
  let len = Array.length x in
  let p = len / 2 in
  let n0 = 2*n in
  let n1 = (2*n + p) mod len in 
  x.(n0) -@ x.(n1)

(* *)
let rec dft_2n x =
  let l = Array.length x in
  let l_h = l / 2 in
  if l = 2 then x.(0) -@ x.(1)
  else
    begin
      let x_even = Array.init l_h (fun n -> x.(2*n)) in
      let y_even = dft_2n x_even in
      let x_odd = Array.init l_h (fun n -> x.(2*n+1)) in
      let y_odd = dft_2n x_odd in
      y_even +@ (y_odd *@ (exp_2pi l (-1))) 
    end


(* *)
let calc_p3 x n =
  let len = Array.length x in
  let p = len / 3 in
  let n0 = 3*n in
  let n1 = (3*n + p) mod len in 
  let n2 = (3*n + 2*p) mod len in
  x.(n0) +@ (x.(n1) *@ (exp_2pi 3 (-1))) +@ (x.(n2) *@ (exp_2pi 3 (-2)))

let calc_p5 x n =
  let len = Array.length x in
  let p = len / 5 in
  let n0 = 5*n in
  let n1 = (5*n + p) mod len in
  let n2 = (5*n + 2*p) mod len in
  let n3 = (5*n + 3*p) mod len in
  let n4 = (5*n + 4*p) mod len in
  x.(n0) +@ (x.(n1) *@ (exp_2pi 5 (-1))) +@ (x.(n2) *@ (exp_2pi 5 (-2))) +@ (x.(n3) *@ (exp_2pi 5 (-3))) +@ (x.(n4) *@ (exp_2pi 5 (-4))) 

let calc_p7 x n =
  let len = Array.length x in
  let p = len / 7 in
  let n0 = 7*n in
  let n1 = (7*n + p) mod len in
  let n2 = (7*n + 2*p) mod len in
  let n3 = (7*n + 3*p) mod len in
  let n4 = (7*n + 4*p) mod len in
  let n5 = (7*n + 5*p) mod len in
  let n6 = (7*n + 6*p) mod len in
  x.(n0) +@ (x.(n1) *@ (exp_2pi 7 (-1))) +@ (x.(n2) *@ (exp_2pi 7 (-2))) +@ (x.(n3) *@ (exp_2pi 7 (-3))) +@ (x.(n4) *@ (exp_2pi 7 (-4))) +@ (x.(n5) *@ (exp_2pi 7 (-5))) +@ (x.(n6) *@ (exp_2pi 7 (-6)))

let dft_6 x =
  let y2 = Array.init 3 (calc_p2 x) in
  calc_p3 y2 0

let dft_10 x =
  let y2 = Array.init 5 (calc_p2 x) in
  calc_p5 y2 0

let dft_30 x =
  let y2 = Array.init 15 (calc_p2 x) in
  let y3 = Array.init 5 (calc_p3 y2) in
  calc_p5 y3 0

let dft_210 x =
  let y2 = Array.init 105 (calc_p2 x) in
  let y3 = Array.init 35 (calc_p3 y2) in
  let y5 = Array.init 7 (calc_p5 y3) in
  calc_p7 y5 0

let calc_210 x n =
  let l = Array.length x in
  let x_210 = Array.init 210 (fun k -> x.(210*k + n) *@ (exp_2pi l (-n))) in
  dft_210 x_210

let dft_44100 n x =
  let xn = Array.make 44100 Complex.zero in
  for i = 0 to (44100-1) do
    xn.((i*n) mod 44100) <- xn.((i*n) mod 44100) +@ x.(i);
  done;
  let y_210 = Array.init 210 (calc_210 xn) in
  (* print_endline (string_of_int n); *)
  Array.fold_left ( +@ ) Complex.zero y_210


let () =
  let x_time = Array.init 44100 (exp_2pi 44100) in
  let z = Array.init 44100 (fun n -> dft_44100 n x_time) in
  ()
