(*
 *
 *)



let init_gl width height =
  GlDraw.shade_model `smooth;
  GlClear.color (0.0, 0.0, 0.0);
  GlClear.depth 1.0;
  GlClear.clear [`color; `depth];
  Gl.enable `depth_test;
  GlFunc.depth_func `lequal;
  GlMisc.hint `perspective_correction `nicest

let z = 0.2
let pi = 4. *. atan 1.

let theta = ref 0.
let omega = 0.02 *. pi

    
let draw_gl_scene () =
  let rec draw_cylinder y0 y1 r0 r1 th0 n idx =
    let s = 2. *. pi *. (float idx) /. (float n) in
    let (x0, z0) = (r0 *. cos (s +. th0), r0 *. sin (s +. th0)) in
    let (x1, z1) = (r1 *. cos (s +. th0), r1 *. sin (s +. th0)) in
    if idx >= n then
      begin
	GlDraw.vertex3(x0, y0, z0);
	GlDraw.vertex3(x1, y1, z1);
	GlDraw.ends()
      end
    else
      begin
	if idx = 0 then
	  begin
	    GlDraw.begins `quad_strip;
	    GlDraw.color(0.7, 0.7, 0.7);
	  end
	else if idx = 2 then GlDraw.color (0.5, 0.5, 0.5);
	GlDraw.vertex3(x0, y0, z0);
	GlDraw.vertex3(x1, y1, z1);
	draw_cylinder y0 y1 r0 r1 th0 n (idx+1)
      end
  in

  let rect t =
    if t <= 0.5 *. pi then t
    else if 0.5 < t && t <= pi then pi -. t
    else 0.
  in
	
  let rec draw_cube (x0, y0, z0) lx ly lz =
    GlDraw.begins `quads;
    
    GlDraw.color (0.5,0.5,0.5);                
    GlDraw.vertex3 (x0+.lx, y0+.ly, z0-.lz);
    GlDraw.vertex3 (x0-.lx, y0+.ly, z0-.lz);
    GlDraw.vertex3 (x0-.lx, y0+.ly, z0-.lz);
    GlDraw.vertex3 (x0+.lx, y0+.ly, z0-.lz);  
    
    GlDraw.color (0.5,0.5,0.5);      
    GlDraw.vertex3 (x0+.lx, y0-.ly, z0+.lz);
    GlDraw.vertex3 (x0-.lx, y0-.ly, z0+.z);
    GlDraw.vertex3 (x0-.lx, y0-.ly, z0-.z);
    GlDraw.vertex3 (x0+.lx, y0-.ly, z0-.z);  
    
    GlDraw.color (0.5,0.5,0.5);
    GlDraw.vertex3 (x0+.lx, y0+.ly, z0+.lz);
    GlDraw.vertex3 (x0-.lx, y0+.ly, z0+.lz);
    GlDraw.vertex3 (x0-.lx, y0-.ly, z0+.lz);
    GlDraw.vertex3 (x0+.lx, y0-.ly, z0+.lz);
    
    GlDraw.color (0.5,0.5,0.5);
    GlDraw.vertex3 (x0+.lx, y0-.ly, z0-.lz); 
    GlDraw.vertex3 (x0-.lx, y0-.ly, z0-.lz); 
    GlDraw.vertex3 (x0-.lx, y0+.ly, z0-.lz); 
    GlDraw.vertex3 (x0+.lx, y0+.ly, z0-.lz); 
    
    GlDraw.color (0.5,0.5,0.5);                   
    GlDraw.vertex3 (x0-.lx, y0+.ly, z0+.lz); 
    GlDraw.vertex3 (x0-.lx, y0+.ly, z0-.lz); 
    GlDraw.vertex3 (x0-.lx, y0-.ly, z0-.lz); 
    GlDraw.vertex3 (x0-.lx, y0-.ly, z0+.lz); 
    
    GlDraw.color (0.5,0.5,0.5);         
    GlDraw.vertex3 (x0+.lx, y0+.ly, z0-.lz); 
    GlDraw.vertex3 (x0+.lx, y0+.ly, z0+.lz); 
    GlDraw.vertex3 (x0+.lx, y0-.ly, z0+.lz); 
    GlDraw.vertex3 (x0+.lx, y0-.ly, z0-.lz); 
    GlDraw.ends ()
  in
  
  let dy = 0.5 *. cos !theta in
	
  GlClear.clear [`color; `depth];
  
  (* Draw cylinder *)
  GlMat.load_identity ();
  GlMat.translate3 (0., 0., -7.0);
  GlMat.rotate3 45.0 (0., 1., 0.);
  draw_cube (2.,4.,-2.) 1. 1. 1.;
  draw_cylinder (3.+.dy) (1.+.dy) 0.5 0.5 !theta 30 0;
  draw_cylinder (1.+.dy) (0.5+.dy) 0.3 0.2 !theta 30 0; 
  
  draw_cube (1.,-.1., (0.5 *. (cos !theta) +. 0.5)) 1. 0.2 1.;
  Glut.swapBuffers ();
  theta := !theta +. omega


(* Handle window reshape events *)
let reshape_cb ~w ~h =
  let ratio = (float_of_int w) /. (float_of_int h) in
  GlDraw.viewport 0 0 w h;
  GlMat.mode `projection;
  GlMat.load_identity ();
  GluMat.perspective 45.0 ratio (0.1, 100.0);
  GlMat.mode `modelview;
  GlMat.load_identity ()

(* Handle keyboard events *)
let keyboard_cb ~key ~x ~y =
  match key with
  | 27 (* ESC *) -> exit 0
  | _ -> ()

(* Draw the scene whever idle *)
let idle_cb () =
  draw_gl_scene ()

let () =
  let width = 640 and height = 480 in
  ignore (Glut.init Sys.argv);
  Glut.initDisplayMode ~alpha:true ~depth:true ~double_buffer:true ();
  Glut.initWindowSize width height;
  ignore (Glut.createWindow "LablGL Machine Tool Simulation");
  Glut.displayFunc draw_gl_scene;
  Glut.keyboardFunc keyboard_cb;
  Glut.reshapeFunc reshape_cb;
  Glut.idleFunc(Some idle_cb);
  init_gl width height;
  Glut.mainLoop ()

