
(*
 * Newton-Lapson 法 : プログラミング in OCaml から
 *)

(* 数値微分 *)
let deriv f =
  let dx = 0.1e-10 in
  fun x -> (f(x+.dx) -. f(x)) /. dx

(* f(a) = a となる a を計算する *)
let fixpoint f x0 =
  let eps = 0.1e-10 in
  let rec eval x =
    let y = f x in
    if abs_float (x -. y) < eps then x
    else eval y
  in eval x0

(* g(x) := x - f(x) / f'(x) *)
let newton_transform f = fun x -> x -. f(x) /. (deriv f x)
let newton_method f x0 = fixpoint (newton_transform f) x0


(*
 * 4 自由度運動方程式の共振周波数の計算
 * f(p) = (α - mp^2)(δ + I_p ωp - Ip^2) - γ^2
 *)
let solve_qdof ~m0 ~i0 ~ip ~alpha ~gamma ~delta ~w =
  let pz = sqrt (alpha /. m0) in
  let fq x =
    let par = alpha -. m0 *. x *. x in
    let rot = delta +. ip *. w *. x -. i0 *. x *. x in
    par *. rot -. gamma *. gamma
  in

  if abs_float gamma < 1.0e-3 then (pz, 0., -0., -.pz)
  else
    begin
      let pz1 = newton_method fq (pz +. 0.01) in
      let pz2 = newton_method fq (0.01) in
      let pz3 = newton_method fq (-0.01) in
      let pz4 = newton_method fq (-.pz -. 0.01) in
      (pz1, pz2, pz3, pz4)
    end

