
(*
 tensor product
  
 *)

(* 1 階テンソルの和 *)
Definition add_r1 {F : Type} (add : F->F->F) (u v : F*F) : F*F :=
  let (u1, u2) := u in
  let (v1, v2) := v in
  (add u1 v1, add u2 v2).

(* 1 階テンソルの F 倍 *)
Definition mul_r1 {F : Type} (mul : F->F->F) (c : F) (u : F*F) : F*F :=
  let (u1, u2) := u in
  (mul u1 c, mul u2 c).

(* テンソル積で 2 階テンソルを得る計算 *)
Definition tensor_r2 {F : Type} (mul : F->F->F) (u v : F*F) : (F*F)*(F*F) :=
  let (u1, u2) := u in
  let (v1, v2) := v in
  (((mul u1 v1), (mul u2 v1)), ((mul u1 v2), (mul u2 v2))).

Lemma tensor_mul_r2 {F : Type} : forall (u1 u2 v1 v2 c : F) (mul : F->F->F), tensor_r2 mul (u1,u2) (mul_r1 mul c (v1, v2)) = (((mul u1 (mul v1 c)), (mul u2 (mul v1 c))), ((mul u1 (mul v2 c)), (mul u2 (mul v2 c)))).
Proof.
  intros; simpl; now reflexivity.
Qed.

Definition add_r2 {F : Type} (add : F->F->F) (u v : (F*F)*(F*F)) : (F*F)*(F*F) :=
  let (u1, u2) := u in
  let (u11, u12) := u1 in
  let (u21, u22) := u2 in
  let (v1, v2) := v in
  let (v11, v12) := v1 in
  let (v21, v22) := v2 in
  ((add u11 v11, add u12 v12), (add u21 v21, add u22 v22)).

Definition mul_r2 {F : Type} (mul : F->F->F) (c : F) (u : (F*F)*(F*F)) : (F*F)*(F*F) :=
  let (u1, u2) := u in
  let (u11, u12) := u1 in
  let (u21, u22) := u2 in
  ((mul c u11, mul c u12), (mul c u21, mul c u22)).


Extraction "tensorproduct.ml" add_r1 mul_r1 tensor_r2 add_r2 mul_r2.